-----------------------------------------------------------------------
-- Locale

local TOOLTIP = {
	-- DPS icon tooltip
	{"DPS", "Target someone and |cffeda55fclick|r, or |cffeda55fdrag and drop|r this icon to assign them as DPS."},
	-- Healer icon tooltip
	{"Healer", "Target someone and |cffeda55fclick|r, or |cffeda55fdrag and drop|r this icon to assign them as a Healer."},
	-- Frame tooltip
	{"Conditioner",
	 "Spot at-a-glance if your raid is ready for a new pull based on total raid health, healer mana and DPS mana.",
	 "Use the icons on the left side of the window to assign people as healers or DPS."},
}

local DRAG_MOVE = "Click and drag to move."
local ROLE_NAMES = {
	"DPS",
	"Healer",
}
local ROLE_ASSIGNED = "%s assigned as %s."
local UNIT_TOOLTIP = "Role: |cffffffff%s|r (assigned)"
local UNIT_TOOLTIP_DEFAULT = "Role: |cffffffff%s|r (default)"

-----------------------------------------------------------------------
-- Locals

-- When these conditions are met, the icon will change to the green checkmark.
-- Health/mana values represent total raid percents.
local READY_HP = 90
local READY_HEALER_MP = 70
local READY_DPS_MP = 70

local hexColors = {
	WTF = "|cffa0a0a0"
}
for k, v in pairs(RAID_CLASS_COLORS) do
	hexColors[k] = "|cff" .. string.format("%02x%02x%02x", v.r * 255, v.g * 255, v.b * 255)
end
local coloredName = setmetatable({}, {__index =
	function(self, key)
		if type(key) == "nil" then return nil end
		local class = select(2, UnitClass(key)) or "WTF"
		self[key] = hexColors[class]  .. key .. "|r"
		return self[key]
	end
})

local savedRoles = nil

local ROLE_DPS = 1
local ROLE_HEALER = 2

local roles = {
	PALADIN = ROLE_HEALER,
	PRIEST = ROLE_HEALER,
	SHAMAN = ROLE_HEALER,
	DRUID = ROLE_DPS,
	MAGE = ROLE_DPS,
	WARLOCK = ROLE_DPS,
	WARRIOR = ROLE_DPS,
	HUNTER = ROLE_DPS,
	ROGUE = ROLE_DPS,
}

-- Functions to update the text labels.
local updateHealth, updateHealerMana, updateDPSMana
-- Function to set the icon.
local setIcon

local shouldUpdateHealth = nil
local shouldUpdateMana = nil

local totalHealth = 0
local totalHealerMana = 0
local totalDPSMana = 0

-----------------------------------------------------------------------
-- Addon and event handling
local frame = CreateFrame("Button", "Conditioner", UIParent)

function frame:ADDON_LOADED(addon)
	if addon ~= "Conditioner" then return end

	if not ConditionerDB then ConditionerDB = {} end
	local realm = GetRealmName()
	if not ConditionerDB[realm] then ConditionerDB[realm] = {} end
	local faction = UnitFactionGroup("player")
	if not ConditionerDB[realm][faction] then ConditionerDB[realm][faction] = {} end
	savedRoles = ConditionerDB[realm][faction]
end

local function getRole(name)
	if savedRoles and savedRoles[name] then
		return savedRoles[name]
	else
		local class = select(2, UnitClass(name))
		return roles[class]
	end
end

do
	local total = 0
	frame:SetScript("OnUpdate", function(self, elapsed)
		total = total + elapsed
		if total > 2 then
			if shouldUpdateHealth or shouldUpdateMana then
				local num = GetNumRaidMembers()
				local health, healerMana, dpsMana, numTotal, numHealers, numDPS, dead = 0, 0, 0, 0, 0, 0, 0
				for i = 1, num do
					local n, _, group, _, _, _, _, _, isDead = GetRaidRosterInfo(i)
					if n and group < 6 then
						if isDead then dead = dead + 1 end
						numTotal = numTotal + 1
						if shouldUpdateHealth then
							health = health + math.min((UnitHealth(n) / UnitHealthMax(n)) * 100, 100)
						end
						if shouldUpdateMana and UnitPowerType(n) == 0 then
							local role = getRole(n)
							local m = math.min((UnitMana(n) / UnitManaMax(n)) * 100, 100)
							if role == ROLE_DPS then
								dpsMana = dpsMana + m
								numDPS = numDPS + 1
							elseif role == ROLE_HEALER then
								healerMana = healerMana + m
								numHealers = numHealers + 1
							end
						end
					end
				end
				if shouldUpdateHealth then
					totalHealth = math.floor(health / numTotal)
					updateHealth(totalHealth)
					shouldUpdateHealth = nil
				end
				if shouldUpdateMana then
					totalHealerMana = math.floor(healerMana / numHealers)
					updateHealerMana(totalHealerMana)
					totalDPSMana = math.floor(dpsMana / numDPS)
					updateDPSMana(totalDPSMana)
					shouldUpdateMana = nil
				end
				setIcon(dead == 0 and totalHealth > READY_HP and totalHealerMana > READY_HEALER_MP and totalDPSMana > READY_DPS_MP)
			end
			total = 0
		end
	end)
end

do
	local inGroup = nil
	function frame:RAID_ROSTER_UPDATE()
		local num = GetNumRaidMembers()
		if num > 0 and not inGroup then
			inGroup = true
			frame:RegisterEvent("UNIT_HEALTH")
			frame:RegisterEvent("UNIT_MANA")
			frame:Show()
		elseif num == 0 and inGroup then
			inGroup = nil
			frame:UnregisterEvent("UNIT_HEALTH")
			frame:UnregisterEvent("UNIT_MANA")
			frame:Hide()
		end
	end
end

function frame:UNIT_HEALTH(unit)
	if UnitInRaid(unit) then shouldUpdateHealth = true end
end
function frame:UNIT_MANA(unit)
	if UnitInRaid(unit) then shouldUpdateMana = true end
end

frame:SetScript("OnEvent", function(self, event, ...)
	self[event](self, ...)
end)
frame:RegisterEvent("RAID_ROSTER_UPDATE")
frame:RegisterEvent("ADDON_LOADED")

local origSetUnit = GameTooltip:GetScript("OnTooltipSetUnit")
GameTooltip:SetScript("OnTooltipSetUnit", function(tooltip, ...)
	local name, unit = tooltip:GetUnit()
	if frame:IsShown() and UnitInRaid(unit) then
		local r = savedRoles[name]
		if r then
			tooltip:AddLine(UNIT_TOOLTIP:format(ROLE_NAMES[r]))
		else
			local class = select(2, UnitClass(unit))
			if class and roles[class] then
				tooltip:AddLine(UNIT_TOOLTIP_DEFAULT:format(ROLE_NAMES[roles[class]]))
			end
		end
	end
	return origSetUnit(tooltip, ...)
end)

-----------------------------------------------------------------------
-- Set up the Conditioner frame

local function tooltip(this)
	if InCombatLockdown() then return end
	local id = this:GetID()
	GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
	for i, v in pairs(TOOLTIP[id]) do
		GameTooltip:AddLine(v, nil, nil, nil, 1)
	end
	GameTooltip:AddLine(DRAG_MOVE, .5, .5, .5, 1)
	GameTooltip:Show()
end
local function clearTooltip()
	GameTooltip:Hide()
end

local function barOnMouseDown()
	this:StartMoving()
end
local function barOnHide()
	this:StopMovingOrSizing()
end

frame:SetClampedToScreen(true)
frame:SetHeight(38)
frame:SetFrameStrata("LOW")
frame:SetToplevel(1)
frame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 150, -150)
frame:SetBackdrop({
	bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
	edgeSize = 8,
	insets = {left = 2, right = 2, top = 2, bottom = 2}
})
frame:SetBackdropBorderColor(1, 1, 1)
frame:SetBackdropColor(0.094, 0.094, 0.094)

frame:SetMovable(true)
frame:RegisterForDrag("LeftButton")
frame:SetScript("OnDragStart", barOnMouseDown)
frame:SetScript("OnDragStop", barOnHide)
frame:SetScript("OnEnter", tooltip)
frame:SetScript("OnLeave", clearTooltip)
frame:SetID(3)

local ICON_READY = "Interface\\RaidFrame\\ReadyCheck-Ready"
local ICON_NOT_READY = "Interface\\RaidFrame\\ReadyCheck-NotReady"
local readyIcon = frame:CreateTexture(nil, "OVERLAY")
readyIcon:SetWidth(32)
readyIcon:SetHeight(32)
readyIcon:ClearAllPoints()
readyIcon:SetPoint("TOPLEFT", frame, "TOPLEFT", 4, -4)
readyIcon:SetTexture(ICON_NOT_READY)
setIcon = function(ready)
	readyIcon:SetTexture(ready and ICON_READY or ICON_NOT_READY)
end

local font = GameFontNormal:GetFont()
local health = frame:CreateFontString(nil, "OVERLAY")
health:SetFont(font, 11)
health:SetVertexColor(0, 1, 0)
health:SetHeight(10)
health:SetText("Health")
health:ClearAllPoints()
health:SetPoint("TOPLEFT", frame, "TOPLEFT", 36, -4)
local healthValue = frame:CreateFontString(nil, "OVERLAY")
healthValue:SetFont(font, 11)
healthValue:SetHeight(10)
healthValue:SetText("100%")
healthValue:SetJustifyH("RIGHT")
healthValue:ClearAllPoints()
healthValue:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -4, -4)
updateHealth = function(input)
	healthValue:SetText(input .. "%")
end

local healerMana = frame:CreateFontString(nil, "OVERLAY")
healerMana:SetFont(font, 11)
healerMana:SetVertexColor(0.2, 0.1, 1)
healerMana:SetHeight(10)
healerMana:SetText("Healer")
healerMana:ClearAllPoints()
healerMana:SetPoint("TOPLEFT", frame, "TOPLEFT", 36, -14)
local healerManaValue = frame:CreateFontString(nil, "OVERLAY")
healerManaValue:SetFont(font, 11)
healerManaValue:SetHeight(10)
healerManaValue:SetText("100%")
healerManaValue:SetJustifyH("RIGHT")
healerManaValue:ClearAllPoints()
healerManaValue:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -4, -14)
updateHealerMana = function(input)
	healerManaValue:SetText(input .. "%")
end

local dpsMana = frame:CreateFontString(nil, "OVERLAY")
dpsMana:SetFont(font, 11)
dpsMana:SetVertexColor(0, 0.1, 0.8)
dpsMana:SetHeight(10)
dpsMana:SetText("DPS")
dpsMana:ClearAllPoints()
dpsMana:SetPoint("TOPLEFT", frame, "TOPLEFT", 36, -24)
local dpsManaValue = frame:CreateFontString(nil, "OVERLAY")
dpsManaValue:SetFont(font, 11)
dpsManaValue:SetHeight(10)
dpsManaValue:SetText("100%")
dpsManaValue:SetJustifyH("RIGHT")
dpsManaValue:ClearAllPoints()
dpsManaValue:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -4, -24)
updateDPSMana = function(input)
	dpsManaValue:SetText(input .. "%")
end

local maxLabelWidth = math.max(health:GetStringWidth(), healerMana:GetStringWidth(), dpsMana:GetStringWidth())
local maxValueWidth = math.max(healthValue:GetStringWidth(), healerManaValue:GetStringWidth(), dpsManaValue:GetStringWidth())
frame:SetWidth(maxLabelWidth + maxValueWidth + 52)

-----------------------------------------------------------------------
-- Create the role icons

local function onUpdate()
	local x, y = GetCursorPosition()
	local s = frame:GetEffectiveScale()
	this:ClearAllPoints()
	this:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", (x / s ) + 24, (y / s) - 24)
end
local function onDragStart()
	this:SetScript("OnUpdate", onUpdate)
end
local function onDragStop()
	this:SetScript("OnUpdate", nil)
	this:GetScript("OnMouseUp")()
end
local originalPoints = {
	{"BOTTOMRIGHT", frame, "BOTTOMLEFT", 0, 3},
	{"TOPRIGHT", frame, "TOPLEFT", 0, -3},
}
local function onMouseUp()
	local i = this:GetID()
	this:ClearAllPoints()
	this:SetPoint(unpack(originalPoints[i]))
	local unit = nil
	if UnitExists("mouseover") then
		unit = "mouseover"
	elseif UnitExists("target") then
		unit = "target"
	end
	if not unit then return end
	local n = UnitName(unit)
	DEFAULT_CHAT_FRAME:AddMessage(ROLE_ASSIGNED:format(coloredName[n], ROLE_NAMES[i]))
	savedRoles[n] = i
end

local healer = CreateFrame("Button", "ConditionerHealerButton", frame)
healer:SetMovable(true)
healer:SetWidth(16)
healer:SetHeight(16)
healer:SetPoint("TOPRIGHT", frame, "TOPLEFT", 0, -3)
healer:RegisterForDrag("LeftButton")
healer:SetScript("OnDragStart", onDragStart)
healer:SetScript("OnDragStop", onDragStop)
healer:SetScript("OnMouseUp", onMouseUp)
healer:SetScript("OnClick", onClick)
healer:SetScript("OnEnter", tooltip)
healer:SetScript("OnLeave", clearTooltip)
healer:SetID(ROLE_HEALER)
local healerIcon = healer:CreateTexture("ConditionerHealerButtonIcon", "ARTWORK")
healerIcon:SetTexture("Interface\\Icons\\Spell_Holy_ChampionsGrace")
healerIcon:SetAllPoints()

local dps = CreateFrame("Button", "ConditionerDPSButton", frame)
dps:SetMovable(true)
dps:SetWidth(16)
dps:SetHeight(16)
dps:SetPoint("BOTTOMRIGHT", frame, "BOTTOMLEFT", 0, 3)
dps:RegisterForDrag("LeftButton")
dps:SetScript("OnDragStart", onDragStart)
dps:SetScript("OnDragStop", onDragStop)
dps:SetScript("OnMouseUp", onMouseUp)
dps:SetScript("OnClick", onClick)
dps:SetScript("OnEnter", tooltip)
dps:SetScript("OnLeave", clearTooltip)
dps:SetID(ROLE_DPS)
local dpsIcon = dps:CreateTexture("ConditionerDPSButtonIcon", "ARTWORK")
dpsIcon:SetTexture("Interface\\Icons\\Spell_Holy_ChampionsBond")
dpsIcon:SetAllPoints()

frame:Hide()

